@@ -18,20 +18,20 @@
 #include "config.h"
 #include "gf-icon-view.h"
 
-#include <gdk/gdkx.h>
 #include <glib/gi18n.h>
 
+#include "dbus/gf-file-manager-gen.h"
+#include "dbus/gf-nautilus-gen.h"
 #include "gf-create-folder-dialog.h"
 #include "gf-desktop-enum-types.h"
 #include "gf-desktop-enums.h"
 #include "gf-dummy-icon.h"
-#include "gf-file-manager-gen.h"
 #include "gf-home-icon.h"
 #include "gf-icon.h"
 #include "gf-monitor-view.h"
-#include "gf-nautilus-gen.h"
 #include "gf-trash-icon.h"
 #include "gf-utils.h"
+#include "gf-workarea-watcher.h"
 
 typedef struct
 {
@@ -44,6 +44,8 @@ struct _GfIconView
 {
   GtkEventBox         parent;
 
+  GfWorkareaWatcher  *workarea_watcher;
+
   GfThumbnailFactory *thumbnail_factory;
 
   GtkGesture         *multi_press;
@@ -92,6 +94,8 @@ enum
   UNSELECT_ALL,
 
   ACTIVATE,
+
+  TRASH,
   DELETE,
 
   TOGGLE,
@@ -1721,7 +1725,8 @@ find_monitor_view_by_monitor (GfIconView *self,
 }
 
 static void
-workarea_changed (GfIconView *self)
+workarea_watcher_changed_cb (GfWorkareaWatcher *workarea_watcher,
+                             GfIconView        *self)
 {
   GList *children;
   GList *l;
@@ -1801,11 +1806,25 @@ monitor_removed_cb (GdkDisplay *display,
                     GfIconView *self)
 {
   GtkWidget *view;
+  GList *l;
 
   view = find_monitor_view_by_monitor (self, monitor);
   if (view == NULL)
     return;
 
+  for (l = self->icons; l != NULL; l = l->next)
+    {
+      GfIconInfo *info;
+
+      info = (GfIconInfo *) l->data;
+
+      if (info->view != view)
+        continue;
+
+      gf_monitor_view_remove_icon (GF_MONITOR_VIEW (view), info->icon);
+      info->view = NULL;
+    }
+
   gtk_widget_destroy (view);
 }
 
@@ -1921,55 +1940,6 @@ show_trash_changed_cb (GSettings  *settings,
     }
 }
 
-static GdkFilterReturn
-filter_func (GdkXEvent *xevent,
-             GdkEvent  *event,
-             gpointer   user_data)
-{
-  XEvent *x;
-  GdkAtom atom;
-
-  x = (XEvent *) xevent;
-
-  if (x->type != PropertyNotify)
-    return GDK_FILTER_CONTINUE;
-
-  atom = gdk_atom_intern_static_string ("_NET_WORKAREA");
-  if (x->xproperty.atom == gdk_x11_atom_to_xatom (atom))
-    workarea_changed (GF_ICON_VIEW (user_data));
-
-  return GDK_FILTER_CONTINUE;
-}
-
-static void
-remove_event_filter (GfIconView *self)
-{
-  GdkScreen *screen;
-  GdkWindow *root;
-
-  screen = gtk_widget_get_screen (GTK_WIDGET (self));
-  root = gdk_screen_get_root_window (screen);
-
-  gdk_window_remove_filter (root, filter_func, self);
-}
-
-static void
-add_event_filter (GfIconView *self)
-{
-  GdkScreen *screen;
-  GdkWindow *root;
-  GdkEventMask event_mask;
-
-  screen = gtk_widget_get_screen (GTK_WIDGET (self));
-  root = gdk_screen_get_root_window (screen);
-
-  event_mask = gdk_window_get_events (root);
-  event_mask |= GDK_PROPERTY_NOTIFY;
-
-  gdk_window_add_filter (root, filter_func, self);
-  gdk_window_set_events (root, event_mask);
-}
-
 static char **
 get_selected_uris (GfIconView *self)
 {
@@ -2034,8 +2004,8 @@ activate_cb (GfIconView *self,
 }
 
 static void
-delete_cb (GfIconView *self,
-           gpointer    user_data)
+trash_cb (GfIconView *self,
+          gpointer    user_data)
 {
   gboolean can_delete;
   GList *l;
@@ -2065,6 +2035,12 @@ delete_cb (GfIconView *self,
   g_strfreev (uris);
 }
 
+static void
+delete_cb (GfIconView *self,
+           gpointer    user_data)
+{
+}
+
 static void
 toggle_cb (GfIconView *self,
            gpointer    user_data)
@@ -2399,6 +2375,8 @@ gf_icon_view_dispose (GObject *object)
 
   self = GF_ICON_VIEW (object);
 
+  g_clear_object (&self->workarea_watcher);
+
   g_clear_object (&self->thumbnail_factory);
 
   g_clear_object (&self->multi_press);
@@ -2442,8 +2420,6 @@ gf_icon_view_finalize (GObject *object)
 
   g_clear_pointer (&self->dummy_icon, gtk_widget_unparent);
 
-  remove_event_filter (self);
-
   G_OBJECT_CLASS (gf_icon_view_parent_class)->finalize (object);
 }
 
@@ -2618,6 +2594,10 @@ install_signals (void)
                   G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
                   0, NULL, NULL, NULL, G_TYPE_NONE, 0);
 
+  view_signals[TRASH] =
+    g_signal_new ("trash", GF_TYPE_ICON_VIEW,
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                  0, NULL, NULL, NULL, G_TYPE_NONE, 0);
 
   view_signals[DELETE] =
     g_signal_new ("delete", GF_TYPE_ICON_VIEW,
@@ -2686,13 +2666,23 @@ add_bindings (GtkBindingSet *binding_set)
   gtk_binding_entry_add_signal (binding_set, GDK_KEY_KP_Enter, modifiers,
                                 "activate", 0);
 
-  /* Delete */
+  /* Trash */
 
   modifiers = 0;
   gtk_binding_entry_add_signal (binding_set, GDK_KEY_Delete, modifiers,
-                                "delete", 0);
+                                "trash", 0);
 
   modifiers = 0;
+  gtk_binding_entry_add_signal (binding_set, GDK_KEY_KP_Delete, modifiers,
+                                "trash", 0);
+
+  /* Delete */
+
+  modifiers = GDK_SHIFT_MASK;
+  gtk_binding_entry_add_signal (binding_set, GDK_KEY_Delete, modifiers,
+                                "delete", 0);
+
+  modifiers = GDK_SHIFT_MASK;
   gtk_binding_entry_add_signal (binding_set, GDK_KEY_KP_Delete, modifiers,
                                 "delete", 0);
 
@@ -2771,11 +2761,16 @@ gf_icon_view_init (GfIconView *self)
   g_signal_connect (self, "select-all", G_CALLBACK (select_all_cb), NULL);
   g_signal_connect (self, "unselect-all", G_CALLBACK (unselect_all_cb), NULL);
   g_signal_connect (self, "activate", G_CALLBACK (activate_cb), NULL);
+  g_signal_connect (self, "trash", G_CALLBACK (trash_cb), NULL);
   g_signal_connect (self, "delete", G_CALLBACK (delete_cb), NULL);
   g_signal_connect (self, "toggle", G_CALLBACK (toggle_cb), NULL);
   g_signal_connect (self, "move", G_CALLBACK (move_cb), NULL);
 
-  add_event_filter (self);
+  self->workarea_watcher = gf_workarea_watcher_new ();
+
+  g_signal_connect (self->workarea_watcher, "changed",
+                    G_CALLBACK (workarea_watcher_changed_cb),
+                    self);
 
   self->thumbnail_factory = gf_thumbnail_factory_new ();
 
