@@ -17,6 +17,7 @@
  */
 
 #include "config.h"
+#include "gf-input-source-manager.h"
 
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
@@ -26,10 +27,10 @@
 #include <X11/Xatom.h>
 
 #include "gf-ibus-manager.h"
-#include "gf-input-source.h"
-#include "gf-input-source-manager.h"
+#include "gf-input-source-ibus.h"
 #include "gf-input-source-popup.h"
 #include "gf-input-source-settings.h"
+#include "gf-input-source-xkb.h"
 #include "gf-keyboard-manager.h"
 
 #define DESKTOP_WM_KEYBINDINGS_SCHEMA "org.gnome.desktop.wm.keybindings"
@@ -43,9 +44,6 @@ struct _SourceInfo
   gchar *type;
 
   gchar *id;
-
-  gchar *display_name;
-  gchar *short_name;
 };
 
 typedef struct
@@ -156,6 +154,7 @@ get_active_window (void)
 
   gdk_x11_display_error_trap_push (display);
 
+  prop = NULL;
   status = XGetWindowProperty (xdisplay,
                                DefaultRootWindow (xdisplay),
                                _net_active_window,
@@ -169,21 +168,12 @@ get_active_window (void)
                                &bytes_after,
                                &prop);
 
+  gdk_x11_display_error_trap_pop_ignored (display);
+
   if (status != Success ||
       actual_type != XA_WINDOW)
     {
-      if (prop)
-        XFree (prop);
-
-      gdk_x11_display_error_trap_pop_ignored (display);
-
-      return None;
-    }
-
-  if (gdk_x11_display_error_trap_pop (display) != Success)
-    {
-      if (prop)
-        XFree (prop);
+      XFree (prop);
 
       return None;
     }
@@ -395,9 +385,7 @@ get_symbol_from_char_code (gunichar code)
 
 static SourceInfo *
 source_info_new (const gchar *type,
-                 const gchar *id,
-                 const gchar *display_name,
-                 const gchar *short_name)
+                 const gchar *id)
 {
   SourceInfo *info;
 
@@ -405,8 +393,6 @@ source_info_new (const gchar *type,
 
   info->type = g_strdup (type);
   info->id = g_strdup (id);
-  info->display_name = g_strdup (display_name);
-  info->short_name = g_strdup (short_name);
 
   return info;
 }
@@ -420,8 +406,6 @@ source_info_free (gpointer data)
 
   g_free (info->type);
   g_free (info->id);
-  g_free (info->display_name);
-  g_free (info->short_name);
 
   g_free (info);
 }
@@ -517,7 +501,6 @@ modifiers_accelerator_activated_cb (GfKeybindings    *keybindings,
 {
   GfInputSourceManager *manager;
   guint size;
-  GList *keys;
 
   manager = GF_INPUT_SOURCE_MANAGER (user_data);
 
@@ -528,28 +511,9 @@ modifiers_accelerator_activated_cb (GfKeybindings    *keybindings,
       return TRUE;
     }
 
-  keys = g_hash_table_get_keys (manager->input_sources);
-  keys = g_list_sort (keys, compare_indexes);
-
   if (keybinding_type == GF_KEYBINDING_ISO_NEXT_GROUP)
     {
-      GfInputSource *current_source;
-      GfInputSource *next_source;
-      guint next_index;
-
-      current_source = manager->current_source;
-      if (current_source == NULL)
-        current_source = g_hash_table_lookup (manager->input_sources,
-                                              GUINT_TO_POINTER (0));
-
-      next_index = gf_input_source_get_index (current_source ) + 1;
-      if (next_index > GPOINTER_TO_UINT (g_list_nth_data (keys, size - 1)))
-        next_index = 0;
-
-      next_source = g_hash_table_lookup (manager->input_sources,
-                                         GUINT_TO_POINTER (next_index));
-
-      gf_input_source_activate (next_source, TRUE);
+      gf_input_source_manager_active_next_source (manager);
     }
   else if (keybinding_type == GF_KEYBINDING_ISO_FIRST_GROUP)
     {
@@ -562,16 +526,19 @@ modifiers_accelerator_activated_cb (GfKeybindings    *keybindings,
     }
   else if (keybinding_type == GF_KEYBINDING_ISO_LAST_GROUP)
     {
+      GList *keys;
       GfInputSource *last_source;
 
+      keys = g_hash_table_get_keys (manager->input_sources);
+      keys = g_list_sort (keys, compare_indexes);
+
       last_source = g_hash_table_lookup (manager->input_sources,
                                          g_list_nth_data (keys, size - 1));
 
       gf_input_source_activate (last_source, TRUE);
+      g_list_free (keys);
     }
 
-  g_list_free (keys);
-
   return TRUE;
 }
 
@@ -668,41 +635,6 @@ keybindings_init (GfInputSourceManager *manager)
   switch_input_backward_changed_cb (manager->wm_keybindings, NULL, manager);
 }
 
-static gchar *
-make_engine_short_name (IBusEngineDesc *engine_desc)
-{
-  const gchar *symbol;
-  const gchar *language;
-
-  symbol = ibus_engine_desc_get_symbol (engine_desc);
-
-  if (symbol != NULL && symbol[0] != '\0')
-    return g_strdup (symbol);
-
-  language = ibus_engine_desc_get_language (engine_desc);
-
-  if (language != NULL && language[0] != '\0')
-    {
-      gchar **codes;
-
-      codes = g_strsplit (language, "_", 2);
-
-      if (strlen (codes[0]) == 2 || strlen (codes[0]) == 3)
-        {
-          gchar *short_name;
-
-          short_name = g_ascii_strdown (codes[0], -1);
-          g_strfreev (codes);
-
-          return short_name;
-        }
-
-      g_strfreev (codes);
-    }
-
-  return get_symbol_from_char_code (0x2328);
-}
-
 static GList *
 get_source_info_list (GfInputSourceManager *manager)
 {
@@ -730,24 +662,16 @@ get_source_info_list (GfInputSourceManager *manager)
       if (g_strcmp0 (type, INPUT_SOURCE_TYPE_XKB) == 0)
         {
           gboolean exists;
-          const gchar *display_name;
-          const gchar *short_name;
 
-          exists = gnome_xkb_info_get_layout_info (xkb_info, id, &display_name,
-                                                   &short_name, NULL, NULL);
+          exists = gnome_xkb_info_get_layout_info (xkb_info, id, NULL,
+                                                   NULL, NULL, NULL);
 
           if (exists)
-            info = source_info_new (type, id, display_name, short_name);
+            info = source_info_new (type, id);
         }
       else if (g_strcmp0 (type, INPUT_SOURCE_TYPE_IBUS) == 0)
         {
           IBusEngineDesc *engine_desc;
-          const gchar *language_code;
-          const gchar *language;
-          const gchar *longname;
-          const gchar *textdomain;
-          gchar *display_name;
-          gchar *short_name;
 
           if (manager->disable_ibus)
             continue;
@@ -758,21 +682,7 @@ get_source_info_list (GfInputSourceManager *manager)
           if (engine_desc == NULL)
             continue;
 
-          language_code = ibus_engine_desc_get_language (engine_desc);
-          language = ibus_get_language_name (language_code);
-          longname = ibus_engine_desc_get_longname (engine_desc);
-          textdomain = ibus_engine_desc_get_textdomain (engine_desc);
-
-          if (*textdomain != '\0' && *longname != '\0')
-            longname = g_dgettext (textdomain, longname);
-
-          display_name = g_strdup_printf ("%s (%s)", language, longname);
-          short_name = make_engine_short_name (engine_desc);
-
-          info = source_info_new (type, id, display_name, short_name);
-
-          g_free (display_name);
-          g_free (short_name);
+          info = source_info_new (type, id);
         }
 
       if (info != NULL)
@@ -785,22 +695,15 @@ get_source_info_list (GfInputSourceManager *manager)
     {
       const gchar *type;
       const gchar *id;
-      const gchar *display_name;
-      const gchar *short_name;
       SourceInfo *info;
 
       type = INPUT_SOURCE_TYPE_XKB;
       id = gf_keyboard_manager_get_default_layout (manager->keyboard_manager);
 
-      gnome_xkb_info_get_layout_info (xkb_info, id, &display_name,
-                                      &short_name, NULL, NULL);
-
-      info = source_info_new (type, id, display_name, short_name);
+      info = source_info_new (type, id);
       list = g_list_append (list, info);
     }
 
-  g_object_unref (xkb_info);
-
   return list;
 }
 
@@ -1189,9 +1092,20 @@ sources_changed_cb (GfInputSourceSettings *settings,
       info = (SourceInfo *) l->data;
       position = g_list_position (source_infos, l);
 
-      source = gf_input_source_new (manager->ibus_manager, info->type,
-                                    info->id, info->display_name,
-                                    info->short_name, position);
+      if (g_strcmp0 (info->type, INPUT_SOURCE_TYPE_IBUS) == 0)
+        {
+          source = gf_input_source_ibus_new (manager->ibus_manager,
+                                             info->id,
+                                             position);
+        }
+      else
+        {
+          GnomeXkbInfo *xkb_info;
+
+          xkb_info = gf_keyboard_manager_get_xkb_info (manager->keyboard_manager);
+
+          source = gf_input_source_xkb_new (xkb_info, info->id, position);
+        }
 
       g_signal_connect (source, "activate",
                         G_CALLBACK (activate_cb), manager);
@@ -1337,10 +1251,11 @@ properties_registered_cb (GfIBusManager *ibus_manager,
   source = (GfInputSource *) g_hash_table_lookup (manager->ibus_sources,
                                                   engine_name);
 
-  if (!source)
+  if (!source || !GF_IS_INPUT_SOURCE_IBUS (source))
     return;
 
-  gf_input_source_set_properties (source, prop_list);
+  gf_input_source_ibus_set_properties (GF_INPUT_SOURCE_IBUS (source),
+                                       prop_list);
 
   if (compare_sources (source, manager->current_source))
     g_signal_emit (manager, signals[SIGNAL_CURRENT_SOURCE_CHANGED], 0, NULL);
@@ -1403,10 +1318,10 @@ property_updated_cb (GfIBusManager *ibus_manager,
   source = (GfInputSource *) g_hash_table_lookup (manager->ibus_sources,
                                                   engine_name);
 
-  if (!source)
+  if (!source || !GF_IS_INPUT_SOURCE_IBUS (source))
     return;
 
-  prop_list = gf_input_source_get_properties (source);
+  prop_list = gf_input_source_ibus_get_properties (GF_INPUT_SOURCE_IBUS (source));
 
   if (!update_sub_property (prop_list, property))
     return;
@@ -1667,3 +1582,36 @@ gf_input_source_manager_get_input_sources (GfInputSourceManager *manager)
 
   return sources;
 }
+
+void
+gf_input_source_manager_active_next_source (GfInputSourceManager *manager)
+{
+  guint size;
+  GList *keys;
+  GfInputSource *current_source;
+  guint next_index;
+  GfInputSource *next_source;
+
+  size = g_hash_table_size (manager->input_sources);
+  if (size == 0)
+    return;
+
+  keys = g_hash_table_get_keys (manager->input_sources);
+  keys = g_list_sort (keys, compare_indexes);
+
+  current_source = manager->current_source;
+
+  if (current_source == NULL)
+    current_source = g_hash_table_lookup (manager->input_sources,
+                                          GUINT_TO_POINTER (0));
+
+  next_index = gf_input_source_get_index (current_source) + 1;
+  if (next_index > GPOINTER_TO_UINT (g_list_nth_data (keys, size - 1)))
+    next_index = 0;
+
+  next_source = g_hash_table_lookup (manager->input_sources,
+                                     GUINT_TO_POINTER (next_index));
+
+  gf_input_source_activate (next_source, TRUE);
+  g_list_free (keys);
+}
